<?php


namespace App\Services\pay;


use Api\PhpServices\Idgen\Idgen;
use App\Models\Dictionary;
use App\Models\order\mysql\PayOrder;
use App\Models\order\mysql\PayOrderClearing;
use App\Models\order\mysql\PayOrderClearingItem;
use App\Services\Order\OrderService;

use Helpers\Strategy;
use Helpers\Logger;

use App\Exception\custom\PayException;

class PayService
{
    // 订单信息
    private $order_info = [];
    // 子订单信息
    private $order_items = [];
    // 分销信息
    private $order_distribution = [];

    private $pay_order = [];
    private $clear_list = [];
    private $clear_items_list = [];


    public function do_pay($order_id, $user_id) {

        // 获取订单信息 + 子单信息 + 分润信息
        $data = OrderService::getOrderData($order_id, $user_id);
        if(empty($data['order'])) {
            throw new PayException(['cus' => 1]);
        }

        // 判断是否存在有效订单
        $pay_order = $this->exist_valid($order_id, $data['order']);

        $this->make_order_clearing($data);

        // 发送ping++ 创建支付订单
        $this->send_pingxx($pay_order);
    }

    /**
     * @param $order_id
     * @param $user_id
     * @throws PayException
     * 判断是否有效订单，如果存在则调用三方判断订单状态
     */
    private function exist_valid($order_id, $order) {

        $expire_time = time() + 30 * 60;
        $pay_order_id = $this->gen_pay_order_id($order_id);
        $pay_order = [
            'pay_order_id'=>$pay_order_id,
            'user_id'=>$order['user_id'],
            'life_account_id'=>$order['life_account_id'],
            'shop_id'=>$order['shop_id'],
            'order_id'=>$order['order_id'],
            'pay_order_status'=>Dictionary::O_PAY_STATUS_INIT,
            'pay_amount'=>$order['payment'],
            'third_order_id'=>'',
            'expire_time'=>$expire_time,
            'is_distribution'=>$order['is_distribution'] ?? 0,
            'source_name'=>1,
            'service_name'=>1,
            'extra'=> json_encode([]),
        ];

        $order = PayOrder::get_valid_order($order_id, $pay_order);
        if(empty($order)) {
            throw new PayException(['cus' => 2]);
            //$this->get_order_from_pingxx($order['third_order_id']);
        }

        //新创建直接返回
        if($order['pay_order_id'] == $pay_order_id) {
            return $order;
        }

        //需要核对ping++ 支付状态
        if(!empty($order['third_order_id'])) {
            $this->get_order_from_pingxx($order['third_order_id']);
            throw new PayException(['cus' => 3]);
        }

        return $order;
    }

    /**
     * @param $data
     * 构造订单流水
     */
    private function make_order_clearing($data) {

        $this->order_info = $data['order'];
        $this->order_items = $data['order_items'];
        $this->order_distribution = $data['order_distribution'];

        $this->clear_list = [];
        $this->clear_items_list = [];

        $wx_tip = $this->make_tip_clearing(); //微信手续费
        $subsidy_tip = $this->make_platform_subsidy_clearing(); //系统补贴
        $merchant_tip = $this->make_merchant_subsidy_clearing(); //商家补贴

        $distribution_tip = $this->make_distribution_clearing(); //团长收益
        $platform_tip = $this->make_platform_clearing(); //平台收益

        $merchant_cash = $this->order_info['payment'] + $subsidy_tip + $merchant_tip - $wx_tip - $distribution_tip - $platform_tip;
        $this->make_merchant_clearing($merchant_cash); //商户收益


        try{
            PayOrderClearing::beginTransaction();
            $cnt = PayOrderClearing::insert($this->clear_list, ['rowCount'=>true]);
            $items_cnt = PayOrderClearingItem::insert($this->clear_items_list, ['rowCount'=>true]);

            if($cnt >0 || $items_cnt>0) {
                PayOrderClearing::commit();
            }
        }catch (\PDOException $e) {
            Logger::error($e->getMessage(), []);
            PayOrderClearing::rollback();
            throw new PayException(['cut'=>5]);
        }
    }

    /**
     * @param $payment
     * @return false|float
     * 获取微信手续费
     */
    private function make_tip_clearing() {
        $total_tip = round(0.006 * $this->order_info['payment']);
        $account = [
            'account_id'=>'100020003001',
            'amount'=>$total_tip,
            'title'=>'微信手续费',
        ];

        if(empty($total_tip)) {
            return 0;
        }

        $account['account_type'] = Dictionary::ACCOUNT_TYPE_ST;
        $account['pay_type'] = Dictionary::PAY_TYPE_IN;
        $account['pay_sub_type'] = Dictionary::PAY_SUB_TYPE_WX;
        $account['need_recorded'] = Dictionary::NO;

        /* 微信手续费总单，在退款、核销的时候会更新这个金额，保证他的准确 */
        $this->do_clearing_data($account, $total_tip, true);
        return $total_tip;
    }

    /**
     * 计算平台运营补贴
     */
    private function make_platform_subsidy_clearing() {
        $account = $this->get_marketing_subsidy();
        $total_tip = $account['amount'];
        if(empty($total_tip)) {
            return 0;
        }

        $account['account_type'] = Dictionary::ACCOUNT_TYPE_SS;
        $account['pay_type'] = Dictionary::PAY_TYPE_OUT;
        $account['pay_sub_type'] = Dictionary::PAY_SUB_TYPE_SYS_OUT;
        $account['need_recorded'] = Dictionary::NO;

        $this->do_clearing_data($account, $total_tip, false);
        return $total_tip;
    }

    /**
     * 计算商户运营补贴
     */
    private function make_merchant_subsidy_clearing() {
        $account = $this->get_marketing_subsidy();
        $total_tip = $account['amount'];
        if(empty($total_tip)) {
            return 0;
        }

        $account['account_type'] = Dictionary::ACCOUNT_TYPE_M;
        $account['pay_type'] = Dictionary::PAY_TYPE_OUT;
        $account['pay_sub_type'] = Dictionary::PAY_SUB_TYPE_M_OUT;
        $account['need_recorded'] = Dictionary::NO;

        $this->do_clearing_data($account, $total_tip, false);
        return $total_tip;
    }

    /**
     * @param $total
     * @return false|float
     * 计算平台收入
     */
    private function make_platform_clearing() {
        $account = [
            'account_id'=>'100020003201',
            'amount'=>0,
            'rate'=>60,
            'title'=>'平台收益',
        ];

        $total_tip = $account['amount'] = round($this->order_info['total_price'] * $account['rate'] / 10000);
        if(empty($total_tip)) {
            return 0;
        }

        $account['account_type'] = Dictionary::ACCOUNT_TYPE_SS;
        $account['pay_type'] = Dictionary::PAY_TYPE_IN;
        $account['pay_sub_type'] = Dictionary::PAY_SUB_TYPE_SYS_IN;
        $account['need_recorded'] = Dictionary::NO;

        $this->do_clearing_data($account, $total_tip, false);
        return $total_tip;
    }

    /**
     * @return mixed
     * 计算分销信息
     */
    private function make_distribution_clearing() {

        if($this->order_info['is_distribution'] == 0) {
            return false;
        }

        /**
        'distributor_user_id'=>122234,
        'distributor_commission_value'=>60,
        'parent_user_id'=>122234 . rand(111, 999),
        'parent_commission_value'=>60,
         */

        $list = [
            [
                'user_id'=>$this->order_distribution['distributor_user_id'],
                'commission_value'=>$this->order_distribution['distributor_commission_value'],
                'commission_type'=>$this->order_distribution['distributor_commission_type'],
            ],
            [
                'user_id'=>$this->order_distribution['parent_user_id'],
                'commission_value'=>$this->order_distribution['parent_commission_value'],
                'commission_type'=>$this->order_distribution['parent_commission_type'],
            ],
        ];

        $total_tip = 0;

        foreach ($list as $account) {

            if($account['commission_type'] == 1) {//rate
                $account['amount'] = round($account['commission_value'] * $this->order_info['total_price'] / 10000);
            } else { //定值
                $account['amount'] = $account['commission_value'];
            }

            $account['account_id'] = $account['account_id'];
            $account['account_type'] = Dictionary::ACCOUNT_TYPE_M;
            $account['pay_type'] = Dictionary::PAY_TYPE_OUT;
            $account['pay_sub_type'] = Dictionary::PAY_SUB_TYPE_M_OUT;
            $account['need_recorded'] = Dictionary::NO;

            $this->do_clearing_data($account, $account['amount'], false);

            $total_tip += $account['amount'];
        }

        return $total_tip;
    }

    /**
     * @param $merchant_cash
     * @return mixed
     * 计算商户清分信息
     */
    private function make_merchant_clearing($merchant_cash) {

        $account = [
            'account_id'=>$this->order_info['life_account_id'],
            'amount'=>$merchant_cash,
            'title'=>'商家订单收益',
        ];

        $account['account_type'] = Dictionary::ACCOUNT_TYPE_M;
        $account['pay_type'] = Dictionary::PAY_TYPE_IN;
        $account['pay_sub_type'] = Dictionary::PAY_SUB_TYPE_M_IN;
        $account['need_recorded'] = Dictionary::NO;

        $this->do_clearing_data($account, $merchant_cash, false);
        return $merchant_cash;

    }

    private function do_clearing_data($account, $total_tip, $is_wx = false) {

        $total_amount = $this->order_info['payment']; //订单支付总价格
        $cleared_amount = 0; //已经结算记录
        $cleared_tip = 0; //已经结算记录

        $pay_order_clearing_id = $this->gen_pay_order_clearing_id();
        $this->clear_list[] = [
            'pay_order_clearing_id' => $pay_order_clearing_id,
            'pay_order_id' => $this->pay_order['pay_order_id'],
            'order_id' => $this->order_info['order_id'],
            'account_id' => $account['account_id'],
            'account_type' => $account['account_type'],
            'pay_amount' => $account['amount'],
            'pay_type' => $account['pay_type'],
            'pay_sub_type' => $account['pay_sub_type'],
            'need_recorded' => $account['need_recorded'],
        ];

        foreach ($this->order_items as $r) {

            if($is_wx) {
                $current_tip = Strategy::getWxTip($r['payment'], $total_amount, $cleared_amount, $total_tip, $cleared_tip);
            } else {
                $current_tip = Strategy::getTip($r['payment'], $total_amount, $cleared_amount, $total_tip, $cleared_tip);
            }

            $this->clear_items_list[] = [
                'pay_order_clearing_item_id' => $this->gen_pay_order_clearing_item_id(),
                'pay_order_clearing_id' => $this->pay_order['pay_order_id'],
                'pay_order_id' => $this->order_info['order_id'],
                'order_id' => $this->order_info['order_id'],
                'account_id' => $account['account_id'],
                'account_type' => $account['account_type'],
                'pay_amount' => $current_tip,
                'pay_type' => $account['pay_type'],
                'pay_sub_type' => $account['pay_sub_type'],
                'need_recorded' => $account['need_recorded'],
            ];

            $cleared_amount += $r['payment'];
            $cleared_tip += $current_tip;
        }
    }


    /**
     * @param $order_id
     * @param $user_id
     * @return string[]
     * 获取支付订单的营销补贴，返回补贴出款账号，补贴金额，补贴原因
     */
    private function get_marketing_subsidy() {
        return [
            'account_id'=>'100020003001',
            'amount'=>1000,
            'title'=>'营销专号支付补贴',
        ];
    }

    /**
     * @param $order
     * @return string[]
     * 获取订单优惠券补贴，一般来自订单，需要这里完善出款账号 + 原因
     */
    private function get_coupon_subsidy($order) {
        return [
            'account_id'=>'100020002001',
            'amount'=>'0',
            'title'=>'营销专号支付补贴',
        ];
    }

    private function gen_pay_order_id($order_id) {
        $number = $order_id;
        return $this->get_idgen_id($number, 1);
    }

    private function gen_pay_order_clearing_id() {
        $number = $this->order_info['order_id'];
        return $this->get_idgen_id($number, 1);
    }

    private function gen_pay_order_clearing_item_id() {
        $number = $this->order_info['order_id'];
        return $this->get_idgen_id($number, 1);
    }

    private function get_idgen_id($number, $count=1){
        $res = Idgen::get(appConfig('idgen.partner'),appConfig('idgen.key'), [], [[
            "type" => "merchant_shop_id",
            'number'=>$number,
            "count"=> $count]]);
        $id = $res['id_datetime']['merchant_shop_id'] ?? [];
        return $id;
    }



    /**
     * @param $data
     * 构造订单支付订单信息
     */
    private function make_new_pay($data) {
        $this->make_new_pay_order($data['order']);
    }
}